@@ -17,7 +17,7 @@
  *  License along with this library; if not, write to the Free
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: shout.c 8808 2005-01-29 19:40:26Z brendan $
+ * $Id: shout.c 10687 2006-01-03 18:56:12Z brendan $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -824,7 +824,7 @@ static int queue_data(shout_queue_t *queue, const unsigned char *data, size_t le
 
 static inline int queue_str(shout_t *self, const char *str)
 {
-	return queue_data(&self->wqueue, str, strlen(str));
+	return queue_data(&self->wqueue, (const unsigned char*)str, strlen(str));
 }
 
 /* this should be shared with sock_write. Create libicecommon. */
@@ -845,12 +845,12 @@ static int queue_printf(shout_t *self, const char *fmt, ...)
 	self->error = SHOUTERR_SUCCESS;
 	if (len > 0) {
 		if ((size_t)len < sizeof(buffer))
-			queue_data(&self->wqueue, buf, len);
+			queue_data(&self->wqueue, (unsigned char*)buf, len);
 		else {
 			buf = malloc(++len);
 			if (buf) {
 				len = vsnprintf(buf, len, fmt, ap_retry);
-				queue_data(&self->wqueue, buf, len);
+				queue_data(&self->wqueue, (unsigned char*)buf, len);
 				free(buf);
 			} else
 				self->error = SHOUTERR_MALLOC;
@@ -890,19 +890,21 @@ static int get_response(shout_t *self)
 	if (rc <= 0)
 		return SHOUTERR_SOCKET;
 
-	if ((rc = queue_data(&self->rqueue, buf, rc)))
+	if ((rc = queue_data(&self->rqueue, (unsigned char*)buf, rc)))
 		return rc;
 
 	/* work from the back looking for \r?\n\r?\n. Anything else means more
 	 * is coming. */
 	for (queue = self->rqueue.head; queue->next; queue = queue->next);
-	pc = queue->data + queue->len - 1;
+	pc = (char*)queue->data + queue->len - 1;
 	blen = queue->len;
 	while (blen) {
 		if (*pc == '\n')
 			newlines++;
+		/* we may have to scan the entire queue if we got a response with
+		 * data after the head line (this can happen with eg 401) */
 		else if (*pc != '\r')
-			break;
+			newlines = 0;
 
 		if (newlines == 2)
 			return SHOUTERR_SUCCESS;
@@ -912,7 +914,7 @@ static int get_response(shout_t *self)
 
 		if (!blen && queue->prev) {
 			queue = queue->prev;
-			pc = queue->data + queue->len - 1;
+			pc = (char*)queue->data + queue->len - 1;
 			blen = queue->len;
 		}
 	}
@@ -1017,7 +1019,7 @@ static int try_write (shout_t *self, const void *data, size_t len)
         if (sock_recoverable (sock_error()))
         {
             self->error = SHOUTERR_BUSY;
-            return 0;
+            return pos;
         }
         self->error = SHOUTERR_SOCKET;
         return ret;
@@ -1181,7 +1183,7 @@ static int parse_response(shout_t *self)
 static int parse_http_response(shout_t *self)
 {
 	http_parser_t *parser;
-	char *header;
+	char *header = NULL;
 	int hlen = 0;
 	int code;
 	char *retcode;
@@ -1198,11 +1200,11 @@ static int parse_http_response(shout_t *self)
 	parser = httpp_create_parser();
 	httpp_initialize(parser, NULL);
 	if (httpp_parse_response(parser, header, hlen, self->mount)) {
-		free (header);
 		retcode = httpp_getvar(parser, HTTPP_VAR_ERROR_CODE);
 		code = atoi(retcode);
 		if(code >= 200 && code < 300) {
 			httpp_destroy(parser);
+			free (header);
 			return SHOUTERR_SUCCESS;
 		}
 	}
@@ -1282,7 +1284,7 @@ static int create_icy_request(shout_t *self)
 			break;
 		if (queue_printf(self, "icy-url:%s\n", self->url ? self->url : "http://www.icecast.org/"))
 			break;
-		if (queue_str(self, "icy-irc:\nicy-aim:icy-icq:\n"))
+		if (queue_str(self, "icy-irc:\nicy-aim:\nicy-icq:\n"))
 			break;
 		if (queue_printf(self, "icy-pub:%i\n", self->public))
 			break;
